package de.intevation.flys.artifacts.model.sq;

import de.intevation.flys.artifacts.access.SQRelationAccess;

import de.intevation.flys.artifacts.model.Calculation;
import de.intevation.flys.artifacts.model.CalculationResult;
import de.intevation.flys.artifacts.model.DateRange;

import de.intevation.flys.backend.SedDBSessionHolder;

import org.apache.log4j.Logger;

public class SQRelationCalculation extends Calculation {

    private static final Logger log =
        Logger.getLogger(SQRelationCalculation.class);

    protected String       river;
    protected double       location;
    protected DateRange [] periods;
    protected double       outliers;

    public SQRelationCalculation() {
    }

    public SQRelationCalculation(SQRelationAccess access) {

        String       river    = access.getRiver();
        Double       location = access.getLocation();
        DateRange [] periods  = access.getPeriods();
        Double       outliers = access.getOutliers();

        if (river == null) {
            // TODO: i18n
            addProblem("sq.missing.river");
        }

        if (location == null) {
            // TODO: i18n
            addProblem("sq.missing.location");
        }

        if (periods == null || periods.length == 0) {
            // TODO: i18n
            addProblem("sq.missing.periods");
        }

        if (outliers == null) {
            // TODO: i18n
            addProblem("sq.missing.outliers");
        }

        if (!hasProblems()) {
            this.river    = river;
            this.location = location;
            this.periods  = periods;
            this.outliers = outliers;
        }
    }


    public CalculationResult calculate() {
        log.debug("SQRelationCalculation.calculate");

        if (hasProblems()) {
            return new CalculationResult(this);
        }

        SedDBSessionHolder.acquire();
        try {
            return internalCalculate();
        }
        finally {
            SedDBSessionHolder.release();
        }
    }

    protected CalculationResult internalCalculate() {

        boolean debug = log.isDebugEnabled();

        // XXX: We can only produce one SQResult.
        DateRange period = periods[0];

        Measurements measurements =
            MeasurementFactory.getMeasurements(river, location, period);

        if (debug) {
            log.debug(measurements.toString());
        }

        SQFractionResult [] fractionResults =
            new SQFractionResult[SQResult.NUMBER_FRACTIONS];

        for (int i = 0; i < fractionResults.length; ++i) {
            // TODO: Implement me!
        }

        return new CalculationResult(new SQResult(fractionResults), this);
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf-8 :
